#include <SPI.h>
#include <Pixy2.h>
#define Enable1 9   //left motor
#define Enable2 3  // right motor
#define Input1 10
#define Input2 4
#define Input4 7
#define Input3 8
#define PING 5

int dist = 11; //stopping distance from wall
double goal = 6; //goal distace from IR sens to wall w prop feedback 
double Kprop = 2; //prop feedback constant

uint16_t blocks;
int32_t x_val;
int map_x_pos = 0;
Pixy2 pixy;



#define IR1 A0 // Pin connected to left IR sens
#define IR2 A1 //Pin connected to right IR sens
// Measure the voltage at 5V and the actual resistance of your
// 47k resistor, and enter them below:

int distance; //returned by measure func
unsigned long pulseduration = 0;

void setup() {
  pixy.init(); //initalize picycam object 
  pixy.ccc.getBlocks(); //get block data from pixycam

  //set all pin modes
  pinMode(Enable1, OUTPUT); //left motor
  pinMode(Enable2, OUTPUT); //right motor
  pinMode(Input1, OUTPUT);
  pinMode(Input2, OUTPUT);
  pinMode(Input3, OUTPUT);
  pinMode(Input4, OUTPUT);
  pinMode(PING, OUTPUT);

  pinMode(IR1, INPUT); //Left IR
  pinMode(IR2, INPUT); //Right IR
  Serial.begin(9600); // Serial Monitor
}

void loop() {
  adjustRight(); //keep righ wall distance
  adjustLeft(); //keep left wall distance
  goForward(); //move forward and read colors
}

// funcion used to read colors while pixy is in "backup" state when stuck
bool pixyCamCommands2() {
  delay(200);
  unsigned long initTime = millis();
  if (pixy.ccc.getBlocks()) {
    int sig = pixy.ccc.blocks[0].m_signature;
    if (sig == 1) {  // one direction point left
      Serial.println("orange");
      while (millis() - initTime < 950) {
        pointTurnLeft();
      }

      sig = 0;
      return true;
    } else if (sig == 2) { // another direction point-turn right
      Serial.println("green");
      while (millis() - initTime < 1000) {
        pointTurnRight();
      }

      return true;
    } else if (sig == 3) {  // u-turn
      Serial.println("purple");
      while (millis() - initTime < 1550) {
        pointTurnLeft();
      }
      return true;
    } else {

    }
  }
}

//function used to read colors when robot is stopped and contains backup function for
// when robot is stopped and sees no color
bool pixyCamCommands() {
  delay(200);
  unsigned long initTime = millis();
  if (pixy.ccc.getBlocks()) {
    int sig = pixy.ccc.blocks[0].m_signature;
    if (sig == 1) {  // one direction point left
      Serial.println("orange");
      while (millis() - initTime < 950) {
        pointTurnLeft();
      }

      sig = 0;
      return true;
    } else if (sig == 2) { // another direction point-turn right
      Serial.println("green");
      while (millis() - initTime < 1000) {
        pointTurnRight();
      }

      return true;
    } else if (sig == 3) {  // u-turn
      Serial.println("purple");
      while (millis() - initTime < 1550) {
        pointTurnLeft();
      }
      return true;
    } else {

    }

  }
  //backup if pixy dosent see color and is stuck
  while (millis() - initTime < 500) {
    goBackward();
  }
  //check for colors after backing up
  if (pixyCamCommands2()) {
    return true;
  }
  //point turn left and check for colors
  if (pixyTurnLeft()) {
    return true;
  }
  //point turn right and check for colors
  if (pixyTurnRight()) {
    return true;
  }  
}

//turns right, stops and looks for colors
bool pixyTurnRight() {
  
  analogWrite(Enable1, 70);
  analogWrite(Enable2, 70);
  digitalWrite(Input1, HIGH);
  digitalWrite(Input2, LOW);
  digitalWrite(Input3, LOW);
  digitalWrite(Input4, HIGH);
  delay(900);
  
  analogWrite(Enable1, 0);
  analogWrite(Enable2, 0);
  digitalWrite(Input1, LOW);
  digitalWrite(Input2, LOW);
  digitalWrite(Input3, LOW);
  digitalWrite(Input4, LOW);
  delay(2000);

  return pixyCamCommands2();
}

//turns right, stops and looks for colors
bool pixyTurnLeft(){
  
  analogWrite(Enable1, 70);
  analogWrite(Enable2, 70);
  digitalWrite(Input1, LOW);
  digitalWrite(Input2, HIGH);
  digitalWrite(Input3, HIGH);
  digitalWrite(Input4, LOW);
  delay(300);
 
  analogWrite(Enable1, 0);
  analogWrite(Enable2, 0);
  digitalWrite(Input1, LOW);
  digitalWrite(Input2, LOW);
  digitalWrite(Input3, LOW);
  digitalWrite(Input4, LOW);
  delay(2000);
  
  return pixyCamCommands2();
  
}
 //calculates and returns distance read by ping sens
void measureDistance()
{
  // set pin as output so we can send a pulse
  pinMode(PING, OUTPUT);
  // set output to LOW
  digitalWrite(PING, LOW);
  delayMicroseconds(5);

  // now send the 5uS pulse out to activate Ping)))
  digitalWrite(PING, HIGH);
  delayMicroseconds(5);
  digitalWrite(PING, LOW);

  // now we need to change the digital pin
  // to input to read the incoming pulse
  pinMode(PING, INPUT);

  // finally, measure the length of the incoming pulse
  pulseduration = pulseIn(PING, HIGH);
  distance = (pulseduration / 29) / 2;
}

//goes forward so long as ping does not read a wall closer than dist. When stopped looks for colors
void goForward() {
  if (distance > dist) {
    analogWrite(Enable1, 70);
    analogWrite(Enable2, 70);
    digitalWrite(Input1, HIGH);
    digitalWrite(Input2, LOW);
    digitalWrite(Input3, HIGH);
    digitalWrite(Input4, LOW);
    delay(30);
  }
  else {
    analogWrite(Enable1, 0);
    analogWrite(Enable2, 0);
    digitalWrite(Input1, LOW);
    digitalWrite(Input2, LOW);
    digitalWrite(Input3, LOW);
    digitalWrite(Input4, LOW);
    delay(3000);
    pixyCamCommands();
  }

}

//goes backwards
void goBackward() {
  analogWrite(Enable1, 70);
  analogWrite(Enable2, 70);
  digitalWrite(Input1, LOW);
  digitalWrite(Input2, HIGH);
  digitalWrite(Input3, LOW);
  digitalWrite(Input4, HIGH);
}

//turns right
void pointTurnRight() {
  analogWrite(Enable1, 70);
  analogWrite(Enable2, 70);
  digitalWrite(Input1, HIGH);
  digitalWrite(Input2, LOW);
  digitalWrite(Input3, LOW);
  digitalWrite(Input4, HIGH);
}

//turns left
void pointTurnLeft() {
  analogWrite(Enable1, 70);
  analogWrite(Enable2, 70);
  digitalWrite(Input1, LOW);
  digitalWrite(Input2, HIGH);
  digitalWrite(Input3, HIGH);
  digitalWrite(Input4, LOW);
}

//reads right wall distance and uses prop feedback to keep a distance greater than "goal"
void adjustLeft() {
  float volts = analogRead(IR1) * (5.0 / 1024.0); // value from sensor * (5/1024)
  float dist1 = 15.037 * pow(volts, -.855);
  float error = dist1 - goal; //calc error
  float errorR = error * Kprop; // mult error by Kprop
  //adjust left if error is greater than .5 and not stopped
  if (errorR > 0.50 && distance > dist) {
    Serial.println("in here adjust right");
    analogWrite(Enable1, 80);
    analogWrite(Enable2, 50);
    digitalWrite(Input1, HIGH);
    digitalWrite(Input2, LOW);
    digitalWrite(Input3, HIGH);
    digitalWrite(Input4, LOW);
    delay(50);
  }
}

void adjustRight() {
  float volts = analogRead(IR2) * (5.0 / 1024.0); // value from sensor * (5/1024)
  float dist2 = 15.037 * pow(volts, -.855);
  float error = dist2 - goal; //calc error
  float errorR = error * Kprop; // mult error by Kprop
  //adjust left if error is greater than .5 and not stopped
  if (errorR > 0.50 && distance > dist) {
    Serial.println("in here adjust left");
    analogWrite(Enable1, 50);
    analogWrite(Enable2, 80);
    digitalWrite(Input1, HIGH);
    digitalWrite(Input2, LOW);
    digitalWrite(Input3, HIGH);
    digitalWrite(Input4, LOW);
    delay(50);
  }
}
